# This file contains the class that holds the results of a shell command.

#--
################################################################################
#                      Copyright (C) 2011 Travis Herrick                       #
################################################################################
#                                                                              #
#                                 \v^V,^!v\^/                                  #
#                                 ~%       %~                                  #
#                                 {  _   _  }                                  #
#                                 (  *   -  )                                  #
#                                 |    /    |                                  #
#                                  \   _,  /                                   #
#                                   \__.__/                                    #
#                                                                              #
################################################################################
# This program is free software: you can redistribute it                       #
# and/or modify it under the terms of the GNU Lesser General Public License    #
# as published by the Free Software Foundation,                                #
# either version 3 of the License, or (at your option) any later version.      #
################################################################################
# This program is distributed in the hope that it will be useful,              #
# but WITHOUT ANY WARRANTY;                                                    #
# without even the implied warranty of MERCHANTABILITY                         #
# or FITNESS FOR A PARTICULAR PURPOSE.                                         #
# See the GNU Lesser General Public License for more details.                  #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.        #
################################################################################
#++

# This class holds the results from an executed shell command.
class ShellResult
  attr_reader :out, :err

  # Constructor
  # ==== Input
  # [out : String] The output from stdout.
  # [err : String] The output from stderr.
  # ==== Output
  # [ShellResult] A new ShellResult object.
  def initialize(out, err)
    @out = out
    @err = err
  end

  # Indicates whether the command was executed successfully.
  #
  # The output is based on the contents of the err attribute.
  # ==== Output
  # [Boolean] Indicates whether the command was successful.
  # ==== Examples
  #  ShellResult.new('a', nil).success?     #=> true
  #  ShellResult.new('a', '').success?      #=> true
  #  ShellResult.new('a', ' ').success?     #=> false
  #  ShellResult.new('a', 'uh-oh').success? #=> false
  def success?
    @err.nil? || @err.empty?
  end

  # Combines the output and error strings.
  # ==== Output
  # [String] Returns the output and error strings concatenated.
  # ==== Examples
  #  ShellResult.new('a', 'b').to_s #=> "a\nb"
  def to_s
    "#{@out}\n#{@err}".strip
  end

  # Cleans up the output string, if necessary.
  # ==== Input
  # [command : String] The command that was called.
  # [path : String] The current directory when the command was run.
  # [out : String] The output (on stdout) from the command.
  # ==== Output
  # [String] A string that has been cleaned up.
  # ==== Notes
  # This is primarily for Windows since cmd adds
  # a bunch of garbage to the output.
  def self.cleanup(command, path, out)
    case Platform::IMPL
      when :mingw
        # Remove all the extra stuff that the cmd prompt adds.
        out.gsub!(/\n\n#{path.gsub(%r[/], '\\\\\\')}>\Z/, '')

        # replace contains the command line prompt
        # and the command up to the first space.
        replace = path.gsub(%r[/], '\\\\\\')
        replace += '>'
        replace += command[0..(command.index(/ /) || 0) - 1]

        # Remove the header portion of the text.
        # This includes the Microsoft 'banner' text
        # that consumes the first two lines.
        out = out.gsub(/\A.+#{replace}.*?$/m, '').strip
    end

    return out
  end
end
